@@ -238,7 +238,7 @@ def to_model(cosmology, *_, method):
     # make Model class
     CosmoModel = type(clsname, (_CosmologyModel,), {**attrs, **params})
     # override __signature__ and format the doc.
-    setattr(CosmoModel.evaluate, "__signature__", msig)
+    CosmoModel.evaluate.__signature__ = msig
     CosmoModel.evaluate.__doc__ = CosmoModel.evaluate.__doc__.format(
         cosmo_cls=cosmo_cls.__qualname__, method=method
     )
