@@ -17,10 +17,11 @@ import pytest
 # LOCAL
 import astropy.cosmology.units as cu
 import astropy.units as u
-from astropy.cosmology import Cosmology, CosmologyError, FlatCosmologyMixin
+from astropy.cosmology import Cosmology, FlatCosmologyMixin
 from astropy.cosmology.core import _COSMOLOGY_CLASSES
 from astropy.cosmology.parameter import Parameter
 from astropy.table import Column, QTable, Table
+from astropy.utils.compat import PYTHON_LT_3_11
 from astropy.utils.exceptions import AstropyDeprecationWarning
 from astropy.utils.metadata import MetaData
 
@@ -31,20 +32,30 @@ from .test_parameter import ParameterTestMixin
 # SETUP / TEARDOWN
 
 
-scalar_zs = [
-    0, 1, 1100,  # interesting times
-    # FIXME! np.inf breaks some funcs. 0 * inf is an error
-    np.float64(3300),  # different type
-    2 * cu.redshift, 3 * u.one  # compatible units
-]
-_zarr = np.linspace(0, 1e5, num=20)
-array_zs = [
-    _zarr,  # numpy
-    _zarr.tolist(),  # pure python
-    Column(_zarr),  # table-like
-    _zarr * cu.redshift  # Quantity
-]
-valid_zs = scalar_zs + array_zs
+def make_valid_zs(max_z: float = 1e5):
+    """Make a list of valid redshifts for testing."""
+    # scalar
+    scalar_zs = [
+        0,
+        1,
+        min(1100, max_z),  # interesting times
+        # FIXME! np.inf breaks some funcs. 0 * inf is an error
+        np.float64(min(3300, max_z)),  # different type
+        2 * cu.redshift,
+        3 * u.one,  # compatible units
+    ]
+    # array
+    _zarr = np.linspace(0, min(1e5, max_z), num=20)
+    array_zs = [
+        _zarr,  # numpy
+        _zarr.tolist(),  # pure python
+        Column(_zarr),  # table-like
+        _zarr * cu.redshift,  # Quantity
+    ]
+    return scalar_zs, _zarr, array_zs, scalar_zs + array_zs
+
+
+scalar_zs, z_arr, array_zs, valid_zs = make_valid_zs()
 
 invalid_zs = [
     (None, TypeError),  # wrong type
@@ -61,7 +72,7 @@ class SubCosmology(Cosmology):
     Tcmb0 = Parameter(unit=u.K)
     m_nu = Parameter(unit=u.eV)
 
-    def __init__(self, H0, Tcmb0=0*u.K, m_nu=0*u.eV, name=None, meta=None):
+    def __init__(self, H0, Tcmb0=0 * u.K, m_nu=0 * u.eV, name=None, meta=None):
         super().__init__(name=name, meta=meta)
         self.H0 = H0
         self.Tcmb0 = Tcmb0
@@ -94,31 +105,23 @@ class MetaTestMixin:
         cosmo.meta[key] = cosmo.meta.pop(key)  # will error if immutable
 
 
-class TestCosmology(ParameterTestMixin, MetaTestMixin,
-                    ReadWriteTestMixin, ToFromFormatTestMixin,
-                    metaclass=abc.ABCMeta):
-    """Test :class:`astropy.cosmology.Cosmology`.
-
-    Subclasses should define tests for:
-
-    - ``test_clone_change_param()``
-    - ``test_repr()``
+class CosmologyTest(
+    ParameterTestMixin,
+    MetaTestMixin,
+    ReadWriteTestMixin,
+    ToFromFormatTestMixin,
+    metaclass=abc.ABCMeta,
+):
+    """
+    Test subclasses of :class:`astropy.cosmology.Cosmology`.
     """
 
+    @abc.abstractmethod
     def setup_class(self):
-        """
-        Setup for testing.
-        Cosmology should not be instantiated, so tests are done on a subclass.
-        """
-        # make sure SubCosmology is known
-        _COSMOLOGY_CLASSES["SubCosmology"] = SubCosmology
-
-        self.cls = SubCosmology
-        self._cls_args = dict(H0=70 * (u.km / u.s / u.Mpc), Tcmb0=2.7 * u.K, m_nu=0.6 * u.eV)
-        self.cls_kwargs = dict(name=self.__class__.__name__, meta={"a": "b"})
+        """Setup for testing."""
 
     def teardown_class(self):
-        _COSMOLOGY_CLASSES.pop("SubCosmology", None)
+        pass
 
     @property
     def cls_args(self):
@@ -151,6 +154,7 @@ class TestCosmology(ParameterTestMixin, MetaTestMixin,
 
     def test_init_subclass(self, cosmo_cls):
         """Test creating subclasses registers classes and manages Parameters."""
+
         class InitSubclassTest(cosmo_cls):
             pass
 
@@ -172,9 +176,13 @@ class TestCosmology(ParameterTestMixin, MetaTestMixin,
 
         # test matches __init__, but without 'self'
         sig = inspect.signature(cosmo.__init__)  # (instances don't have self)
-        assert set(sig.parameters.keys()) == set(cosmo._init_signature.parameters.keys())
-        assert all(np.all(sig.parameters[k].default == p.default) for k, p in
-                   cosmo._init_signature.parameters.items())
+        assert set(sig.parameters.keys()) == set(
+            cosmo._init_signature.parameters.keys()
+        )
+        assert all(
+            np.all(sig.parameters[k].default == p.default)
+            for k, p in cosmo._init_signature.parameters.items()
+        )
 
     # ---------------------------------------------------------------
     # instance-level
@@ -197,13 +205,17 @@ class TestCosmology(ParameterTestMixin, MetaTestMixin,
         assert cosmo.name == self.cls_kwargs["name"]  # test has expected value
 
         # immutable
-        with pytest.raises(AttributeError, match="can't set"):
+        match = (
+            "can't set"
+            if PYTHON_LT_3_11
+            else f"property 'name' of {cosmo.__class__.__name__!r} object has no setter"
+        )
+        with pytest.raises(AttributeError, match=match):
             cosmo.name = None
 
+    @abc.abstractmethod
     def test_is_flat(self, cosmo_cls, cosmo):
-        """Test property ``is_flat``. It's an ABC."""
-        with pytest.raises(NotImplementedError, match="is_flat is not implemented"):
-            cosmo.is_flat
+        """Test property ``is_flat``."""
 
     # ------------------------------------------------
     # clone
@@ -293,22 +305,21 @@ class TestCosmology(ParameterTestMixin, MetaTestMixin,
         # class in string rep
         assert cosmo_cls.__qualname__ in r
         assert r.index(cosmo_cls.__qualname__) == 0  # it's the first thing
-        r = r[len(cosmo_cls.__qualname__) + 1:]  # remove
+        r = r[len(cosmo_cls.__qualname__) + 1 :]  # remove
 
         # name in string rep
         if cosmo.name is not None:
-            assert f"name=\"{cosmo.name}\"" in r
+            assert f'name="{cosmo.name}"' in r
             assert r.index("name=") == 0
-            r = r[6 + len(cosmo.name) + 3:]  # remove
+            r = r[6 + len(cosmo.name) + 3 :]  # remove
 
         # parameters in string rep
         ps = {k: getattr(cosmo, k) for k in cosmo.__parameters__}
-        cps = {k: getattr(cosmo_cls, k) for k in cosmo.__parameters__}
         for k, v in ps.items():
             sv = f"{k}={v}"
             assert sv in r
             assert r.index(k) == 0
-            r = r[len(sv) + 2:]  # remove
+            r = r[len(sv) + 2 :]  # remove
 
     # ------------------------------------------------
 
@@ -366,26 +377,41 @@ class TestCosmology(ParameterTestMixin, MetaTestMixin,
         assert unpickled.meta == cosmo.meta
 
 
-class CosmologySubclassTest(TestCosmology):
-    """
-    Test subclasses of :class:`astropy.cosmology.Cosmology`.
-    This is broken away from ``TestCosmology``, because |Cosmology| is/will be
-    an ABC and subclasses must override some methods.
+class TestCosmology(CosmologyTest):
+    """Test :class:`astropy.cosmology.Cosmology`.
+
+    Subclasses should define tests for:
+
+    - ``test_clone_change_param()``
+    - ``test_repr()``
     """
 
-    @abc.abstractmethod
     def setup_class(self):
-        """Setup for testing."""
+        """
+        Setup for testing.
+        Cosmology should not be instantiated, so tests are done on a subclass.
+        """
+        # make sure SubCosmology is known
+        _COSMOLOGY_CLASSES["SubCosmology"] = SubCosmology
+
+        self.cls = SubCosmology
+        self._cls_args = dict(
+            H0=70 * (u.km / u.s / u.Mpc), Tcmb0=2.7 * u.K, m_nu=0.6 * u.eV
+        )
+        self.cls_kwargs = dict(name=self.__class__.__name__, meta={"a": "b"})
+
+    def teardown_class(self):
+        """Teardown for testing."""
+        super().teardown_class(self)
+        _COSMOLOGY_CLASSES.pop("SubCosmology", None)
 
     # ===============================================================
     # Method & Attribute Tests
 
-    # ---------------------------------------------------------------
-    # instance-level
-
-    @abc.abstractmethod
     def test_is_flat(self, cosmo_cls, cosmo):
-        """Test property ``is_flat``."""
+        """Test property ``is_flat``. It's an ABC."""
+        with pytest.raises(NotImplementedError, match="is_flat is not implemented"):
+            cosmo.is_flat
 
 
 # -----------------------------------------------------------------------------
@@ -406,17 +432,16 @@ class FlatCosmologyMixinTest:
     """
 
     def test_nonflat_class_(self, cosmo_cls, cosmo):
-        """Test :attr:`astropy.cosmology.core.FlatCosmologyMixin.nonflat_cls`.
-        """
+        """Test :attr:`astropy.cosmology.core.FlatCosmologyMixin.nonflat_cls`."""
         # Test it's a method on the class
-        assert issubclass(cosmo_cls, cosmo_cls._nonflat_cls_)
+        assert issubclass(cosmo_cls, cosmo_cls.__nonflatclass__)
 
         # It also works from the instance. # TODO! as a "metaclassmethod"
-        assert issubclass(cosmo_cls, cosmo._nonflat_cls_)
+        assert issubclass(cosmo_cls, cosmo.__nonflatclass__)
 
         # Maybe not the most robust test, but so far all Flat classes have the
         # name of their parent class.
-        assert cosmo._nonflat_cls_.__name__ in cosmo_cls.__name__
+        assert cosmo.__nonflatclass__.__name__ in cosmo_cls.__name__
 
     def test_is_flat(self, cosmo_cls, cosmo):
         """Test property ``is_flat``."""
@@ -426,8 +451,7 @@ class FlatCosmologyMixinTest:
         assert cosmo.is_flat is True
 
     def test_nonflat(self, cosmo):
-        """Test :attr:`astropy.cosmology.core.FlatCosmologyMixin.nonflat`.
-        """
+        """Test :attr:`astropy.cosmology.core.FlatCosmologyMixin.nonflat`."""
         assert cosmo.nonflat.is_equivalent(cosmo)
         assert cosmo.is_equivalent(cosmo.nonflat)
 
@@ -438,7 +462,7 @@ class FlatCosmologyMixinTest:
         """Test method ``.clone()``to_nonflat argument."""
         # just converting the class
         nc = cosmo.clone(to_nonflat=True)
-        assert isinstance(nc, cosmo._nonflat_cls_)
+        assert isinstance(nc, cosmo.__nonflatclass__)
         assert nc == cosmo.nonflat
 
     @abc.abstractmethod
@@ -450,7 +474,7 @@ class FlatCosmologyMixinTest:
         """
         # send to non-flat
         nc = cosmo.clone(to_nonflat=True)
-        assert isinstance(nc, cosmo._nonflat_cls_)
+        assert isinstance(nc, cosmo.__nonflatclass__)
         assert nc == cosmo.nonflat
 
     # ------------------------------------------------
@@ -463,7 +487,10 @@ class FlatCosmologyMixinTest:
         e.g. FlatFLRWMixinTest -> FlatCosmologyMixinTest -> TestCosmology
         vs   FlatFLRWMixinTest -> FlatCosmologyMixinTest -> TestFLRW -> TestCosmology
         """
-        CosmologySubclassTest.test_is_equivalent(self, cosmo)
+        CosmologyTest.test_is_equivalent(self, cosmo)
+
+        # See FlatFLRWMixinTest for tests. It's a bit hard here since this class
+        # is for an ABC.
 
     # ===============================================================
     # Usage Tests
@@ -475,7 +502,7 @@ class FlatCosmologyMixinTest:
             pass
 
         # The classes have the same non-flat parent class
-        assert SubClass1._nonflat_cls_ is cosmo_cls._nonflat_cls_
+        assert SubClass1.__nonflatclass__ is cosmo_cls.__nonflatclass__
 
         # A more complex example is when Mixin classes are used.
         class Mixin:
@@ -485,25 +512,25 @@ class FlatCosmologyMixinTest:
             pass
 
         # The classes have the same non-flat parent class
-        assert SubClass2._nonflat_cls_ is cosmo_cls._nonflat_cls_
+        assert SubClass2.__nonflatclass__ is cosmo_cls.__nonflatclass__
 
         # The order of the Mixin should not matter
         class SubClass3(cosmo_cls, Mixin):
             pass
 
         # The classes have the same non-flat parent class
-        assert SubClass3._nonflat_cls_ is cosmo_cls._nonflat_cls_
+        assert SubClass3.__nonflatclass__ is cosmo_cls.__nonflatclass__
 
 
-def test_nonflat_cls_multiple_nonflat_inheritance():
+def test__nonflatclass__multiple_nonflat_inheritance():
     """
-    Test :meth:`astropy.cosmology.core.FlatCosmologyMixin._nonflat_cls_`
+    Test :meth:`astropy.cosmology.core.FlatCosmologyMixin.__nonflatclass__`
     when there's more than one non-flat class in the inheritance.
     """
+
     # Define a non-operable minimal subclass of Cosmology.
     class SubCosmology2(Cosmology):
-
-        def __init__(self, H0, Tcmb0=0*u.K, m_nu=0*u.eV, name=None, meta=None):
+        def __init__(self, H0, Tcmb0=0 * u.K, m_nu=0 * u.eV, name=None, meta=None):
             super().__init__(name=name, meta=meta)
 
         @property
@@ -514,7 +541,6 @@ def test_nonflat_cls_multiple_nonflat_inheritance():
     with pytest.raises(TypeError, match="cannot create a consistent non-flat class"):
 
         class FlatSubCosmology(FlatCosmologyMixin, SubCosmology, SubCosmology2):
-
             @property
             def nonflat(self):
                 pass
