@@ -1,18 +1,21 @@
 # Licensed under a 3-clause BSD style license
 
 import os
-from setuptools import Extension
 
 import numpy
+from setuptools import Extension
 
 ROOT = os.path.relpath(os.path.dirname(__file__))
 
 
 def get_extensions():
-    sources = [os.path.join(ROOT, 'cparser.pyx'),
-               os.path.join(ROOT, 'src', 'tokenizer.c')]
+    sources = [
+        os.path.join(ROOT, "cparser.pyx"),
+        os.path.join(ROOT, "src", "tokenizer.c"),
+    ]
     ascii_ext = Extension(
         name="astropy.io.ascii.cparser",
         include_dirs=[numpy.get_include()],
-        sources=sources)
+        sources=sources,
+    )
     return [ascii_ext]
