@@ -1278,18 +1278,18 @@ class _NonLinearLSQFitter(metaclass=_FitterMeta):
         """
         MESSAGE = "Non-Finite input data has been removed by the fitter."
 
-        if z is None:
-            mask = np.isfinite(y)
-            if not np.all(mask):
-                warnings.warn(MESSAGE, AstropyUserWarning)
-            z_out = None
-        else:
-            mask = np.isfinite(z)
-            if not np.all(mask):
-                warnings.warn(MESSAGE, AstropyUserWarning)
-            z_out = z[mask]
+        mask = np.ones_like(x, dtype=bool) if weights is None else np.isfinite(weights)
+        mask &= np.isfinite(y) if z is None else np.isfinite(z)
 
-        return x[mask], y[mask], z_out, None if weights is None else weights[mask]
+        if not np.all(mask):
+            warnings.warn(MESSAGE, AstropyUserWarning)
+
+        return (
+            x[mask],
+            y[mask],
+            None if z is None else z[mask],
+            None if weights is None else weights[mask],
+        )
 
     @fitter_unit_support
     def __call__(
