@@ -5,17 +5,21 @@ import itertools
 
 import numpy as np
 
+from astropy import log
+from astropy import units as u
+from astropy.io import fits, registry
+from astropy.utils.decorators import sharedmethod
+from astropy.wcs import WCS
+
 from .compat import NDDataArray
 from .nduncertainty import (
-    StdDevUncertainty, NDUncertainty, VarianceUncertainty, InverseVariance)
-from astropy.io import fits, registry
-from astropy import units as u
-from astropy import log
-from astropy.wcs import WCS
-from astropy.utils.decorators import sharedmethod
+    InverseVariance,
+    NDUncertainty,
+    StdDevUncertainty,
+    VarianceUncertainty,
+)
 
-
-__all__ = ['CCDData', 'fits_ccddata_reader', 'fits_ccddata_writer']
+__all__ = ["CCDData", "fits_ccddata_reader", "fits_ccddata_writer"]
 
 _known_uncertainties = (StdDevUncertainty, VarianceUncertainty, InverseVariance)
 _unc_name_to_cls = {cls.__name__: cls for cls in _known_uncertainties}
@@ -47,17 +51,19 @@ def _arithmetic(op):
     ``multiply`` because only these methods from NDArithmeticMixin are
     overwritten.
     """
+
     def decorator(func):
         def inner(self, operand, operand2=None, **kwargs):
             global _config_ccd_requires_unit
             _config_ccd_requires_unit = False
-            result = self._prepare_then_do_arithmetic(op, operand,
-                                                      operand2, **kwargs)
+            result = self._prepare_then_do_arithmetic(op, operand, operand2, **kwargs)
             # Wrap it again as CCDData so it checks the final unit.
             _config_ccd_requires_unit = True
             return result.__class__(result)
+
         inner.__doc__ = f"See `astropy.nddata.NDArithmeticMixin.{func.__name__}`."
         return sharedmethod(inner)
+
     return decorator
 
 
@@ -65,9 +71,9 @@ def _uncertainty_unit_equivalent_to_parent(uncertainty_type, unit, parent_unit):
     if uncertainty_type is StdDevUncertainty:
         return unit == parent_unit
     elif uncertainty_type is VarianceUncertainty:
-        return unit == (parent_unit ** 2)
+        return unit == (parent_unit**2)
     elif uncertainty_type is InverseVariance:
-        return unit == (1 / (parent_unit ** 2))
+        return unit == (1 / (parent_unit**2))
     raise ValueError(f"unsupported uncertainty type: {uncertainty_type}")
 
 
@@ -130,6 +136,11 @@ class CCDData(NDDataArray):
             If the unit is ``None`` or not otherwise specified it will raise a
             ``ValueError``
 
+    psf : `numpy.ndarray` or None, optional
+        Image representation of the PSF at the center of this image. In order
+        for convolution to be flux-preserving, this should generally be
+        normalized to sum to unity.
+
     Raises
     ------
     ValueError
@@ -180,9 +191,9 @@ class CCDData(NDDataArray):
     """
 
     def __init__(self, *args, **kwd):
-        if 'meta' not in kwd:
-            kwd['meta'] = kwd.pop('header', None)
-        if 'header' in kwd:
+        if "meta" not in kwd:
+            kwd["meta"] = kwd.pop("header", None)
+        if "header" in kwd:
             raise ValueError("can't have both header and meta.")
 
         super().__init__(*args, **kwd)
@@ -236,6 +247,16 @@ class CCDData(NDDataArray):
     def unit(self, value):
         self._unit = u.Unit(value)
 
+    @property
+    def psf(self):
+        return self._psf
+
+    @psf.setter
+    def psf(self, value):
+        if value is not None and not isinstance(value, np.ndarray):
+            raise TypeError("The psf must be a numpy array.")
+        self._psf = value
+
     @property
     def header(self):
         return self._meta
@@ -252,37 +273,47 @@ class CCDData(NDDataArray):
     def uncertainty(self, value):
         if value is not None:
             if isinstance(value, NDUncertainty):
-                if getattr(value, '_parent_nddata', None) is not None:
+                if getattr(value, "_parent_nddata", None) is not None:
                     value = value.__class__(value, copy=False)
                 self._uncertainty = value
             elif isinstance(value, np.ndarray):
                 if value.shape != self.shape:
-                    raise ValueError("uncertainty must have same shape as "
-                                     "data.")
+                    raise ValueError("uncertainty must have same shape as data.")
                 self._uncertainty = StdDevUncertainty(value)
-                log.info("array provided for uncertainty; assuming it is a "
-                         "StdDevUncertainty.")
+                log.info(
+                    "array provided for uncertainty; assuming it is a "
+                    "StdDevUncertainty."
+                )
             else:
-                raise TypeError("uncertainty must be an instance of a "
-                                "NDUncertainty object or a numpy array.")
+                raise TypeError(
+                    "uncertainty must be an instance of a "
+                    "NDUncertainty object or a numpy array."
+                )
             self._uncertainty.parent_nddata = self
         else:
             self._uncertainty = value
 
-    def to_hdu(self, hdu_mask='MASK', hdu_uncertainty='UNCERT',
-               hdu_flags=None, wcs_relax=True,
-               key_uncertainty_type='UTYPE', as_image_hdu=False):
+    def to_hdu(
+        self,
+        hdu_mask="MASK",
+        hdu_uncertainty="UNCERT",
+        hdu_flags=None,
+        wcs_relax=True,
+        key_uncertainty_type="UTYPE",
+        as_image_hdu=False,
+        hdu_psf="PSFIMAGE",
+    ):
         """Creates an HDUList object from a CCDData object.
 
         Parameters
         ----------
-        hdu_mask, hdu_uncertainty, hdu_flags : str or None, optional
+        hdu_mask, hdu_uncertainty, hdu_flags, hdu_psf : str or None, optional
             If it is a string append this attribute to the HDUList as
             `~astropy.io.fits.ImageHDU` with the string as extension name.
             Flags are not supported at this time. If ``None`` this attribute
             is not appended.
-            Default is ``'MASK'`` for mask, ``'UNCERT'`` for uncertainty and
-            ``None`` for flags.
+            Default is ``'MASK'`` for mask, ``'UNCERT'`` for uncertainty,
+            ``'PSFIMAGE'`` for psf, and `None` for flags.
 
         wcs_relax : bool
             Value of the ``relax`` parameter to use in converting the WCS to a
@@ -333,7 +364,7 @@ class CCDData(NDDataArray):
                 dummy_ccd._insert_in_metadata_fits_safe(k, v)
             header = dummy_ccd.header
         if self.unit is not u.dimensionless_unscaled:
-            header['bunit'] = self.unit.to_string()
+            header["bunit"] = self.unit.to_string()
         if self.wcs:
             # Simply extending the FITS header with the WCS can lead to
             # duplicates of the WCS keywords; iterating over the WCS
@@ -358,8 +389,8 @@ class CCDData(NDDataArray):
         if hdu_mask and self.mask is not None:
             # Always assuming that the mask is a np.ndarray (check that it has
             # a 'shape').
-            if not hasattr(self.mask, 'shape'):
-                raise ValueError('only a numpy.ndarray mask can be saved.')
+            if not hasattr(self.mask, "shape"):
+                raise ValueError("only a numpy.ndarray mask can be saved.")
 
             # Convert boolean mask to uint since io.fits cannot handle bool.
             hduMask = fits.ImageHDU(self.mask.astype(np.uint8), name=hdu_mask)
@@ -371,8 +402,9 @@ class CCDData(NDDataArray):
             # No idea how this can be done so only allow StdDevUncertainty.
             uncertainty_cls = self.uncertainty.__class__
             if uncertainty_cls not in _known_uncertainties:
-                raise ValueError('only uncertainties of type {} can be saved.'
-                                 .format(_known_uncertainties))
+                raise ValueError(
+                    f"only uncertainties of type {_known_uncertainties} can be saved."
+                )
             uncertainty_name = _unc_cls_to_name[uncertainty_cls]
 
             hdr_uncertainty = fits.Header()
@@ -382,22 +414,30 @@ class CCDData(NDDataArray):
             # this might be problematic if the Uncertainty has a unit differing
             # from the data so abort for different units. This is important for
             # astropy > 1.2
-            if (hasattr(self.uncertainty, 'unit') and
-                    self.uncertainty.unit is not None):
+            if hasattr(self.uncertainty, "unit") and self.uncertainty.unit is not None:
                 if not _uncertainty_unit_equivalent_to_parent(
-                        uncertainty_cls, self.uncertainty.unit, self.unit):
+                    uncertainty_cls, self.uncertainty.unit, self.unit
+                ):
                     raise ValueError(
-                        'saving uncertainties with a unit that is not '
-                        'equivalent to the unit from the data unit is not '
-                        'supported.')
+                        "saving uncertainties with a unit that is not "
+                        "equivalent to the unit from the data unit is not "
+                        "supported."
+                    )
 
-            hduUncert = fits.ImageHDU(self.uncertainty.array, hdr_uncertainty,
-                                      name=hdu_uncertainty)
+            hduUncert = fits.ImageHDU(
+                self.uncertainty.array, hdr_uncertainty, name=hdu_uncertainty
+            )
             hdus.append(hduUncert)
 
         if hdu_flags and self.flags:
-            raise NotImplementedError('adding the flags to a HDU is not '
-                                      'supported at this time.')
+            raise NotImplementedError(
+                "adding the flags to a HDU is not supported at this time."
+            )
+
+        if hdu_psf and self.psf is not None:
+            # The PSF is an image, so write it as a separate ImageHDU.
+            hdu_psf = fits.ImageHDU(self.psf, name=hdu_psf)
+            hdus.append(hdu_psf)
 
         hdulist = fits.HDUList(hdus)
 
@@ -437,29 +477,28 @@ class CCDData(NDDataArray):
         This addresses that case by checking the length of the ``key`` and
         ``value`` and, if necessary, shortening the key.
         """
-
         if len(key) > 8 and len(value) > 72:
             short_name = key[:8]
-            self.meta[f'HIERARCH {key.upper()}'] = (
-                short_name, f"Shortened name for {key}")
+            self.meta[f"HIERARCH {key.upper()}"] = (
+                short_name,
+                f"Shortened name for {key}",
+            )
             self.meta[short_name] = value
         else:
             self.meta[key] = value
 
     # A dictionary mapping "known" invalid fits unit
-    known_invalid_fits_unit_strings = {'ELECTRONS/S': u.electron/u.s,
-                                       'ELECTRONS': u.electron,
-                                       'electrons': u.electron}
+    known_invalid_fits_unit_strings = {
+        "ELECTRONS/S": u.electron / u.s,
+        "ELECTRONS": u.electron,
+        "electrons": u.electron,
+    }
 
 
 # These need to be importable by the tests...
-_KEEP_THESE_KEYWORDS_IN_HEADER = [
-    'JD-OBS',
-    'MJD-OBS',
-    'DATE-OBS'
-]
-_PCs = set(['PC1_1', 'PC1_2', 'PC2_1', 'PC2_2'])
-_CDs = set(['CD1_1', 'CD1_2', 'CD2_1', 'CD2_2'])
+_KEEP_THESE_KEYWORDS_IN_HEADER = ["JD-OBS", "MJD-OBS", "DATE-OBS"]
+_PCs = {"PC1_1", "PC1_2", "PC2_1", "PC2_2"}
+_CDs = {"CD1_1", "CD1_2", "CD2_1", "CD2_2"}
 
 
 def _generate_wcs_and_update_header(hdr):
@@ -469,23 +508,22 @@ def _generate_wcs_and_update_header(hdr):
 
     Parameters
     ----------
-
     hdr : astropy.io.fits.header or other dict-like
 
     Returns
     -------
-
     new_header, wcs
     """
-
     # Try constructing a WCS object.
     try:
         wcs = WCS(hdr)
     except Exception as exc:
         # Normally WCS only raises Warnings and doesn't fail but in rare
         # cases (malformed header) it could fail...
-        log.info('An exception happened while extracting WCS information from '
-                 'the Header.\n{}: {}'.format(type(exc).__name__, str(exc)))
+        log.info(
+            "An exception happened while extracting WCS information from "
+            "the Header.\n{}: {}".format(type(exc).__name__, str(exc))
+        )
         return hdr, None
     # Test for success by checking to see if the wcs ctype has a non-empty
     # value, return None for wcs if ctype is empty.
@@ -523,20 +561,27 @@ def _generate_wcs_and_update_header(hdr):
     # We need to check for any SIP coefficients that got left behind if the
     # header has SIP.
     if wcs.sip is not None:
-        keyword = '{}_{}_{}'
-        polynomials = ['A', 'B', 'AP', 'BP']
+        keyword = "{}_{}_{}"
+        polynomials = ["A", "B", "AP", "BP"]
         for poly in polynomials:
-            order = wcs.sip.__getattribute__(f'{poly.lower()}_order')
+            order = wcs.sip.__getattribute__(f"{poly.lower()}_order")
             for i, j in itertools.product(range(order), repeat=2):
-                new_hdr.remove(keyword.format(poly, i, j),
-                               ignore_missing=True)
+                new_hdr.remove(keyword.format(poly, i, j), ignore_missing=True)
 
     return (new_hdr, wcs)
 
 
-def fits_ccddata_reader(filename, hdu=0, unit=None, hdu_uncertainty='UNCERT',
-                        hdu_mask='MASK', hdu_flags=None,
-                        key_uncertainty_type='UTYPE', **kwd):
+def fits_ccddata_reader(
+    filename,
+    hdu=0,
+    unit=None,
+    hdu_uncertainty="UNCERT",
+    hdu_mask="MASK",
+    hdu_flags=None,
+    key_uncertainty_type="UTYPE",
+    hdu_psf="PSFIMAGE",
+    **kwd,
+):
     """
     Generate a CCDData object from a FITS file.
 
@@ -579,6 +624,10 @@ def fits_ccddata_reader(filename, hdu=0, unit=None, hdu_uncertainty='UNCERT',
 
         .. versionadded:: 3.1
 
+    hdu_psf : str or None, optional
+        FITS extension from which the psf image should be initialized. If the
+        extension does not exist the psf of the CCDData is `None`.
+
     kwd :
         Any additional keyword parameters are passed through to the FITS reader
         in :mod:`astropy.io.fits`; see Notes for additional discussion.
@@ -590,19 +639,19 @@ def fits_ccddata_reader(filename, hdu=0, unit=None, hdu_uncertainty='UNCERT',
     :mod:`astropy.io.fits` are disabled.
     """
     unsupport_open_keywords = {
-        'do_not_scale_image_data': 'Image data must be scaled.',
-        'scale_back': 'Scale information is not preserved.'
+        "do_not_scale_image_data": "Image data must be scaled.",
+        "scale_back": "Scale information is not preserved.",
     }
     for key, msg in unsupport_open_keywords.items():
         if key in kwd:
-            prefix = f'unsupported keyword: {key}.'
-            raise TypeError(' '.join([prefix, msg]))
+            prefix = f"unsupported keyword: {key}."
+            raise TypeError(" ".join([prefix, msg]))
     with fits.open(filename, **kwd) as hdus:
         hdr = hdus[hdu].header
 
         if hdu_uncertainty is not None and hdu_uncertainty in hdus:
             unc_hdu = hdus[hdu_uncertainty]
-            stored_unc_name = unc_hdu.header.get(key_uncertainty_type, 'None')
+            stored_unc_name = unc_hdu.header.get(key_uncertainty_type, "None")
             # For compatibility reasons the default is standard deviation
             # uncertainty because files could have been created before the
             # uncertainty type was stored in the header.
@@ -618,15 +667,21 @@ def fits_ccddata_reader(filename, hdu=0, unit=None, hdu_uncertainty='UNCERT',
             mask = None
 
         if hdu_flags is not None and hdu_flags in hdus:
-            raise NotImplementedError('loading flags is currently not '
-                                      'supported.')
+            raise NotImplementedError("loading flags is currently not supported.")
+
+        if hdu_psf is not None and hdu_psf in hdus:
+            psf = hdus[hdu_psf].data
+        else:
+            psf = None
 
         # search for the first instance with data if
         # the primary header is empty.
         if hdu == 0 and hdus[hdu].data is None:
             for i in range(len(hdus)):
-                if (hdus.info(hdu)[i][3] == 'ImageHDU' and
-                        hdus.fileinfo(i)['datSpan'] > 0):
+                if (
+                    hdus.info(hdu)[i][3] == "ImageHDU"
+                    and hdus.fileinfo(i)["datSpan"] > 0
+                ):
                     hdu = i
                     comb_hdr = hdus[hdu].header.copy()
                     # Add header values from the primary header that aren't
@@ -636,11 +691,11 @@ def fits_ccddata_reader(filename, hdu=0, unit=None, hdu_uncertainty='UNCERT',
                     log.info(f"first HDU with data is extension {hdu}.")
                     break
 
-        if 'bunit' in hdr:
-            fits_unit_string = hdr['bunit']
+        if "bunit" in hdr:
+            fits_unit_string = hdr["bunit"]
             # patch to handle FITS files using ADU for the unit instead of the
             # standard version of 'adu'
-            if fits_unit_string.strip().lower() == 'adu':
+            if fits_unit_string.strip().lower() == "adu":
                 fits_unit_string = fits_unit_string.lower()
         else:
             fits_unit_string = None
@@ -656,44 +711,62 @@ def fits_ccddata_reader(filename, hdu=0, unit=None, hdu_uncertainty='UNCERT',
                     fits_unit_string = u.Unit(fits_unit_string)
                 except ValueError:
                     raise ValueError(
-                        'The Header value for the key BUNIT ({}) cannot be '
-                        'interpreted as valid unit. To successfully read the '
-                        'file as CCDData you can pass in a valid `unit` '
-                        'argument explicitly or change the header of the FITS '
-                        'file before reading it.'
-                        .format(fits_unit_string))
+                        "The Header value for the key BUNIT ({}) cannot be "
+                        "interpreted as valid unit. To successfully read the "
+                        "file as CCDData you can pass in a valid `unit` "
+                        "argument explicitly or change the header of the FITS "
+                        "file before reading it.".format(fits_unit_string)
+                    )
             else:
-                log.info("using the unit {} passed to the FITS reader instead "
-                         "of the unit {} in the FITS file."
-                         .format(unit, fits_unit_string))
+                log.info(
+                    "using the unit {} passed to the FITS reader instead "
+                    "of the unit {} in the FITS file.".format(unit, fits_unit_string)
+                )
 
         use_unit = unit or fits_unit_string
         hdr, wcs = _generate_wcs_and_update_header(hdr)
-        ccd_data = CCDData(hdus[hdu].data, meta=hdr, unit=use_unit,
-                           mask=mask, uncertainty=uncertainty, wcs=wcs)
+        ccd_data = CCDData(
+            hdus[hdu].data,
+            meta=hdr,
+            unit=use_unit,
+            mask=mask,
+            uncertainty=uncertainty,
+            wcs=wcs,
+            psf=psf,
+        )
 
     return ccd_data
 
 
 def fits_ccddata_writer(
-        ccd_data, filename, hdu_mask='MASK', hdu_uncertainty='UNCERT',
-        hdu_flags=None, key_uncertainty_type='UTYPE', as_image_hdu=False,
-        **kwd):
+    ccd_data,
+    filename,
+    hdu_mask="MASK",
+    hdu_uncertainty="UNCERT",
+    hdu_flags=None,
+    key_uncertainty_type="UTYPE",
+    as_image_hdu=False,
+    hdu_psf="PSFIMAGE",
+    **kwd,
+):
     """
     Write CCDData object to FITS file.
 
     Parameters
     ----------
+    ccd_data : CCDData
+        Object to write.
+
     filename : str
         Name of file.
 
-    hdu_mask, hdu_uncertainty, hdu_flags : str or None, optional
+    hdu_mask, hdu_uncertainty, hdu_flags, hdu_psf : str or None, optional
         If it is a string append this attribute to the HDUList as
         `~astropy.io.fits.ImageHDU` with the string as extension name.
         Flags are not supported at this time. If ``None`` this attribute
         is not appended.
-        Default is ``'MASK'`` for mask, ``'UNCERT'`` for uncertainty and
-        ``None`` for flags.
+        Default is ``'MASK'`` for mask, ``'UNCERT'`` for uncertainty,
+        ``'PSFIMAGE'`` for psf, and `None` for flags.
 
     key_uncertainty_type : str, optional
         The header key name for the class name of the uncertainty (if any)
@@ -723,15 +796,19 @@ def fits_ccddata_writer(
         Saving flags is not supported.
     """
     hdu = ccd_data.to_hdu(
-        hdu_mask=hdu_mask, hdu_uncertainty=hdu_uncertainty,
-        key_uncertainty_type=key_uncertainty_type, hdu_flags=hdu_flags,
-        as_image_hdu=as_image_hdu)
+        hdu_mask=hdu_mask,
+        hdu_uncertainty=hdu_uncertainty,
+        key_uncertainty_type=key_uncertainty_type,
+        hdu_flags=hdu_flags,
+        as_image_hdu=as_image_hdu,
+        hdu_psf=hdu_psf,
+    )
     if as_image_hdu:
         hdu.insert(0, fits.PrimaryHDU())
     hdu.writeto(filename, **kwd)
 
 
 with registry.delay_doc_updates(CCDData):
-    registry.register_reader('fits', CCDData, fits_ccddata_reader)
-    registry.register_writer('fits', CCDData, fits_ccddata_writer)
-    registry.register_identifier('fits', CCDData, fits.connect.is_fits)
+    registry.register_reader("fits", CCDData, fits_ccddata_reader)
+    registry.register_writer("fits", CCDData, fits_ccddata_writer)
+    registry.register_identifier("fits", CCDData, fits.connect.is_fits)
