@@ -408,7 +408,6 @@ class SAMPHubServer:
             process runs in a separated thread. `False` is usually used in a
             Python shell.
         """
-
         if self._is_running:
             raise SAMPHubError("Hub is already running")
 
@@ -441,9 +440,8 @@ class SAMPHubServer:
     @property
     def params(self):
         """
-        The hub parameters (which are written to the logfile)
+        The hub parameters (which are written to the logfile).
         """
-
         params = {}
 
         # Keys required by standard profile
@@ -497,7 +495,6 @@ class SAMPHubServer:
         """
         Stop the current SAMP Hub instance and delete the lock file.
         """
-
         if not self._is_running:
             return
 
@@ -845,9 +842,7 @@ class SAMPHubServer:
         if private_key in self._private_keys:
             client_private_key = self._public_id_to_private_key(client_id)
             log.debug(
-                "get_metadata: private-key = {} client-id = {}".format(
-                    private_key, client_id
-                )
+                f"get_metadata: private-key = {private_key} client-id = {client_id}"
             )
             if client_private_key is not None:
                 if client_private_key in self._metadata:
@@ -927,9 +922,7 @@ class SAMPHubServer:
                     return self._id2mtypes[client_private_key]
                 else:
                     log.debug(
-                        "get_subscriptions: client-id = {} mtypes = missing".format(
-                            client_id
-                        )
+                        f"get_subscriptions: client-id = {client_id} mtypes = missing"
                     )
                     return {}
             else:
@@ -993,7 +986,6 @@ class SAMPHubServer:
         >>> SAMPHubServer.get_mtype_subtypes("samp.app.ping")
         ['samp.app.ping', 'samp.app.*', 'samp.*', '*']
         """
-
         subtypes = []
 
         msubs = mtype.split(".")
@@ -1101,6 +1093,11 @@ class SAMPHubServer:
                             target=self._notify,
                             args=(sender_private_key, _recipient_id, message),
                         )
+        if not recipient_ids:
+            warnings.warn(
+                "No client was able to receive this message",
+                SAMPWarning,
+            )
 
         return recipient_ids
 
@@ -1173,9 +1170,7 @@ class SAMPHubServer:
             if "samp.mtype" not in message:
                 raise SAMPProxyError(
                     3,
-                    "samp.mtype keyword is missing in message tagged as {}".format(
-                        msg_tag
-                    ),
+                    f"samp.mtype keyword is missing in message tagged as {msg_tag}",
                 )
 
             public_id = self._private_keys[private_key][0]
@@ -1260,9 +1255,7 @@ class SAMPHubServer:
 
         try:
             log.debug(
-                "reply {} from {} to {}".format(
-                    counter, responder_public_id, recipient_public_id
-                )
+                f"reply {counter} from {responder_public_id} to {recipient_public_id}"
             )
 
             if recipient_msg_tag == "samp::sync::call":
@@ -1308,7 +1301,6 @@ class SAMPHubServer:
         arg_params : tuple
             Any additional arguments to be passed to the SAMP method
         """
-
         if recipient_private_key is None:
             raise SAMPHubError("Invalid client ID")
 
@@ -1413,7 +1405,7 @@ class SAMPHubServer:
     ):
         self._update_last_activity_time()
 
-        if not client_address[0] in ["localhost", "127.0.0.1"]:
+        if client_address[0] not in ["localhost", "127.0.0.1"]:
             raise SAMPProxyError(403, "Request of registration rejected by the Hub.")
 
         if not origin:
