@@ -11,11 +11,20 @@ are based on reference [1]_, which is also the basis for the R package
 """
 
 import numpy as np
+
 from astropy.units import Quantity
 
-__all__ = ['circmean', 'circstd', 'circvar', 'circmoment', 'circcorrcoef',
-           'rayleightest', 'vtest', 'vonmisesmle']
-__doctest_requires__ = {'vtest': ['scipy']}
+__all__ = [
+    "circmean",
+    "circstd",
+    "circvar",
+    "circmoment",
+    "circcorrcoef",
+    "rayleightest",
+    "vtest",
+    "vonmisesmle",
+]
+__doctest_requires__ = {"vtest": ["scipy"]}
 
 
 def _components(data, p=1, phi=0.0, axis=None, weights=None):
@@ -26,10 +35,10 @@ def _components(data, p=1, phi=0.0, axis=None, weights=None):
     try:
         weights = np.broadcast_to(weights, data.shape)
     except ValueError:
-        raise ValueError('Weights and data have inconsistent shape.')
+        raise ValueError("Weights and data have inconsistent shape.")
 
-    C = np.sum(weights * np.cos(p * (data - phi)), axis)/np.sum(weights, axis)
-    S = np.sum(weights * np.sin(p * (data - phi)), axis)/np.sum(weights, axis)
+    C = np.sum(weights * np.cos(p * (data - phi)), axis) / np.sum(weights, axis)
+    S = np.sum(weights * np.sin(p * (data - phi)), axis) / np.sum(weights, axis)
 
     return C, S
 
@@ -55,7 +64,7 @@ def _length(data, p=1, phi=0.0, axis=None, weights=None):
 
 
 def circmean(data, axis=None, weights=None):
-    """ Computes the circular mean angle of an array of circular data.
+    """Computes the circular mean angle of an array of circular data.
 
     Parameters
     ----------
@@ -97,7 +106,7 @@ def circmean(data, axis=None, weights=None):
 
 
 def circvar(data, axis=None, weights=None):
-    """ Computes the circular variance of an array of circular data.
+    """Computes the circular variance of an array of circular data.
 
     There are some concepts for defining measures of dispersion for circular
     data. The variance implemented here is based on the definition given by
@@ -142,16 +151,17 @@ def circvar(data, axis=None, weights=None):
 
     Notes
     -----
-    The definition used here differs from the one in scipy.stats.circvar.
-    Precisely, Scipy circvar uses an approximation based on the limit of small
-    angles which approaches the linear variance.
+    For Scipy < 1.9.0, ``scipy.stats.circvar`` uses a different
+    definition based on an approximation using the limit of small
+    angles that approaches the linear variance. For Scipy >= 1.9.0,
+    ``scipy.stats.cirvar`` uses a definition consistent with this
+    implementation.
     """
-
     return 1.0 - _length(data, 1, 0.0, axis, weights)
 
 
-def circstd(data, axis=None, weights=None, method='angular'):
-    """ Computes the circular standard deviation of an array of circular data.
+def circstd(data, axis=None, weights=None, method="angular"):
+    """Computes the circular standard deviation of an array of circular data.
 
     The standard deviation implemented here is based on the definitions given
     by [1]_, which is also the same used by the R package 'CirStat' [2]_.
@@ -219,17 +229,17 @@ def circstd(data, axis=None, weights=None, method='angular'):
        Series on Multivariate Analysis, Vol. 5, 2001.
 
     """
-    if method not in ('angular', 'circular'):
+    if method not in ("angular", "circular"):
         raise ValueError("method should be either 'angular' or 'circular'")
 
-    if method == 'angular':
-        return np.sqrt(2. * (1. - _length(data, 1, 0.0, axis, weights)))
+    if method == "angular":
+        return np.sqrt(2.0 * (1.0 - _length(data, 1, 0.0, axis, weights)))
     else:
-        return np.sqrt(-2. * np.log(_length(data, 1, 0.0, axis, weights)))
+        return np.sqrt(-2.0 * np.log(_length(data, 1, 0.0, axis, weights)))
 
 
 def circmoment(data, p=1.0, centered=False, axis=None, weights=None):
-    """ Computes the ``p``-th trigonometric circular moment for an array
+    """Computes the ``p``-th trigonometric circular moment for an array
     of circular data.
 
     Parameters
@@ -279,13 +289,11 @@ def circmoment(data, p=1.0, centered=False, axis=None, weights=None):
     else:
         phi = 0.0
 
-    return _angle(data, p, phi, axis, weights), _length(data, p, phi, axis,
-                                                        weights)
+    return _angle(data, p, phi, axis, weights), _length(data, p, phi, axis, weights)
 
 
-def circcorrcoef(alpha, beta, axis=None, weights_alpha=None,
-                 weights_beta=None):
-    """ Computes the circular correlation coefficient between two array of
+def circcorrcoef(alpha, beta, axis=None, weights_alpha=None, weights_beta=None):
+    """Computes the circular correlation coefficient between two array of
     circular data.
 
     Parameters
@@ -334,7 +342,7 @@ def circcorrcoef(alpha, beta, axis=None, weights_alpha=None,
        Circular Statistics (2001)'". 2015.
        <https://cran.r-project.org/web/packages/CircStats/CircStats.pdf>
     """
-    if(np.size(alpha, axis) != np.size(beta, axis)):
+    if np.size(alpha, axis) != np.size(beta, axis):
         raise ValueError("alpha and beta must be arrays of the same size")
 
     mu_a = circmean(alpha, axis, weights_alpha)
@@ -342,13 +350,13 @@ def circcorrcoef(alpha, beta, axis=None, weights_alpha=None,
 
     sin_a = np.sin(alpha - mu_a)
     sin_b = np.sin(beta - mu_b)
-    rho = np.sum(sin_a*sin_b)/np.sqrt(np.sum(sin_a*sin_a)*np.sum(sin_b*sin_b))
+    rho = np.sum(sin_a * sin_b) / np.sqrt(np.sum(sin_a * sin_a) * np.sum(sin_b * sin_b))
 
     return rho
 
 
 def rayleightest(data, axis=None, weights=None):
-    """ Performs the Rayleigh test of uniformity.
+    """Performs the Rayleigh test of uniformity.
 
     This test is  used to identify a non-uniform distribution, i.e. it is
     designed for detecting an unimodal deviation from uniformity. More
@@ -396,25 +404,28 @@ def rayleightest(data, axis=None, weights=None):
        Uniformity." Lecture Notes, STA 6934/5805. University of Florida, 2007.
     .. [4] D. Wilkie. "Rayleigh Test for Randomness of Circular Data". Applied
        Statistics. 1983.
-       <http://citeseerx.ist.psu.edu/viewdoc/summary?doi=10.1.1.211.4762>
+       <http://wexler.free.fr/library/files/wilkie%20(1983)%20rayleigh%20test%20for%20randomness%20of%20circular%20data.pdf>
     """
     n = np.size(data, axis=axis)
     Rbar = _length(data, 1, 0.0, axis, weights)
-    z = n*Rbar*Rbar
+    z = n * Rbar * Rbar
 
     # see [3] and [4] for the formulae below
     tmp = 1.0
-    if(n < 50):
-        tmp = 1.0 + (2.0*z - z*z)/(4.0*n) - (24.0*z - 132.0*z**2.0 +
-                                             76.0*z**3.0 - 9.0*z**4.0)/(288.0 *
-                                                                        n * n)
+    if n < 50:
+        tmp = (
+            1.0
+            + (2.0 * z - z * z) / (4.0 * n)
+            - (24.0 * z - 132.0 * z**2.0 + 76.0 * z**3.0 - 9.0 * z**4.0)
+            / (288.0 * n * n)
+        )
 
-    p_value = np.exp(-z)*tmp
+    p_value = np.exp(-z) * tmp
     return p_value
 
 
 def vtest(data, mu=0.0, axis=None, weights=None):
-    """ Performs the Rayleigh test of uniformity where the alternative
+    """Performs the Rayleigh test of uniformity where the alternative
     hypothesis H1 is assumed to have a known mean angle ``mu``.
 
     Parameters
@@ -462,32 +473,45 @@ def vtest(data, mu=0.0, axis=None, weights=None):
     try:
         weights = np.broadcast_to(weights, data.shape)
     except ValueError:
-        raise ValueError('Weights and data have inconsistent shape.')
+        raise ValueError("Weights and data have inconsistent shape.")
 
     n = np.size(data, axis=axis)
-    R0bar = np.sum(weights * np.cos(data - mu), axis)/np.sum(weights, axis)
+    R0bar = np.sum(weights * np.cos(data - mu), axis) / np.sum(weights, axis)
     z = np.sqrt(2.0 * n) * R0bar
     pz = norm.cdf(z)
     fz = norm.pdf(z)
     # see reference [3]
-    p_value = 1 - pz + fz*((3*z - z**3)/(16.0*n) +
-                           (15*z + 305*z**3 - 125*z**5 + 9*z**7)/(4608.0*n*n))
+    p_value = (
+        1
+        - pz
+        + fz
+        * (
+            (3 * z - z**3) / (16.0 * n)
+            + (15 * z + 305 * z**3 - 125 * z**5 + 9 * z**7) / (4608.0 * n * n)
+        )
+    )
     return p_value
 
 
 def _A1inv(x):
     # Approximation for _A1inv(x) according R Package 'CircStats'
     # See http://www.scienceasia.org/2012.38.n1/scias38_118.pdf, equation (4)
-    if 0 <= x < 0.53:
-        return 2.0*x + x*x*x + (5.0*x**5)/6.0
-    elif x < 0.85:
-        return -0.4 + 1.39*x + 0.43/(1.0 - x)
-    else:
-        return 1.0/(x*x*x - 4.0*x*x + 3.0*x)
+
+    kappa1 = np.where(
+        np.logical_and(0 <= x, x < 0.53), 2.0 * x + x * x * x + (5.0 * x**5) / 6.0, 0
+    )
+    kappa2 = np.where(
+        np.logical_and(0.53 <= x, x < 0.85), -0.4 + 1.39 * x + 0.43 / (1.0 - x), 0
+    )
+    kappa3 = np.where(
+        np.logical_or(x < 0, 0.85 <= x), 1.0 / (x * x * x - 4.0 * x * x + 3.0 * x), 0
+    )
+
+    return kappa1 + kappa2 + kappa3
 
 
-def vonmisesmle(data, axis=None):
-    """ Computes the Maximum Likelihood Estimator (MLE) for the parameters of
+def vonmisesmle(data, axis=None, weights=None):
+    """Computes the Maximum Likelihood Estimator (MLE) for the parameters of
     the von Mises distribution.
 
     Parameters
@@ -497,6 +521,11 @@ def vonmisesmle(data, axis=None):
         radians whenever ``data`` is ``numpy.ndarray``.
     axis : int, optional
         Axis along which the mle will be computed.
+    weights : numpy.ndarray, optional
+        In case of grouped data, the i-th element of ``weights`` represents a
+        weighting factor for each group such that ``sum(weights, axis)``
+        equals the number of observations. See [1]_, remark 1.4, page 22,
+        for detailed explanation.
 
     Returns
     -------
@@ -522,7 +551,7 @@ def vonmisesmle(data, axis=None):
        Circular Statistics (2001)'". 2015.
        <https://cran.r-project.org/web/packages/CircStats/CircStats.pdf>
     """
-    mu = circmean(data, axis=None)
+    mu = circmean(data, axis=axis, weights=weights)
 
-    kappa = _A1inv(np.mean(np.cos(data - mu), axis))
+    kappa = _A1inv(_length(data, p=1, phi=0.0, axis=axis, weights=weights))
     return mu, kappa
