@@ -3,8 +3,8 @@
 import os
 from os.path import join
 
-from distutils.core import Extension
-
+import numpy
+from setuptools import Extension
 
 BLS_ROOT = os.path.relpath(os.path.dirname(__file__))
 
@@ -16,6 +16,6 @@ def get_extensions():
             join(BLS_ROOT, "bls.c"),
             join(BLS_ROOT, "_impl.pyx"),
         ],
-        include_dirs=["numpy"],
+        include_dirs=[numpy.get_include()],
     )
     return [ext]
