@@ -19,7 +19,8 @@ import numpy as np
 from astropy import config as _config
 from astropy.utils.compat import NUMPY_LT_1_22
 from astropy.utils.data_info import ParentDtypeInfo
-from astropy.utils.exceptions import AstropyDeprecationWarning, AstropyWarning
+from astropy.utils.decorators import deprecated
+from astropy.utils.exceptions import AstropyWarning
 from astropy.utils.misc import isiterable
 
 from .core import (
@@ -61,7 +62,7 @@ _UFUNCS_FILTER_WARNINGS = {np.arcsin, np.arccos, np.arccosh, np.arctanh}
 
 class Conf(_config.ConfigNamespace):
     """
-    Configuration parameters for Quantity
+    Configuration parameters for Quantity.
     """
 
     latex_array_threshold = _config.ConfigItem(
@@ -79,7 +80,7 @@ conf = Conf()
 
 class QuantityIterator:
     """
-    Flat iterator object to iterate over Quantities
+    Flat iterator object to iterate over Quantities.
 
     A `QuantityIterator` iterator is returned by ``q.flat`` for any Quantity
     ``q``.  It allows iterating over the array as if it were a 1-D array,
@@ -216,7 +217,6 @@ class QuantityInfo(QuantityInfoBase):
             Empty instance of this class consistent with ``cols``
 
         """
-
         # Get merged info attributes like shape, dtype, format, description, etc.
         attrs = self.merge_cols_attributes(
             cols, metadata_conflicts, name, ("meta", "format", "description")
@@ -304,7 +304,7 @@ class Quantity(np.ndarray):
 
     ndmin : int, optional
         Specifies the minimum number of dimensions that the resulting array
-        should have.  Ones will be pre-pended to the shape as needed to meet
+        should have.  Ones will be prepended to the shape as needed to meet
         this requirement.  This parameter is ignored if the input is a
         `Quantity` and ``copy=False``.
 
@@ -363,10 +363,9 @@ class Quantity(np.ndarray):
 
         Returns
         -------
-        `typing.Annotated`, `typing_extensions.Annotated`, `astropy.units.Unit`, or `astropy.units.PhysicalType`
+        `typing.Annotated`, `astropy.units.Unit`, or `astropy.units.PhysicalType`
             Return type in this preference order:
-            * if python v3.9+ : `typing.Annotated`
-            * if :mod:`typing_extensions` is installed : `typing_extensions.Annotated`
+            * `typing.Annotated`
             * `astropy.units.Unit` or `astropy.units.PhysicalType`
 
         Raises
@@ -392,8 +391,7 @@ class Quantity(np.ndarray):
         With Python 3.9+ or :mod:`typing_extensions`, |Quantity| types are also
         static-type compatible.
         """
-        # LOCAL
-        from ._typing import HAS_ANNOTATED, Annotated
+        from typing import Annotated
 
         # process whether [unit] or [unit, shape, ptype]
         if isinstance(unit_shape_dtype, tuple):  # unit, shape, dtype
@@ -416,20 +414,11 @@ class Quantity(np.ndarray):
                     "unit annotation is not a Unit or PhysicalType"
                 ) from None
 
-        # Allow to sort of work for python 3.8- / no typing_extensions
-        # instead of bailing out, return the unit for `quantity_input`
-        if not HAS_ANNOTATED:
-            warnings.warn(
-                "Quantity annotations are valid static type annotations only"
-                " if Python is v3.9+ or `typing_extensions` is installed."
-            )
-            return unit
-
         # Quantity does not (yet) properly extend the NumPy generics types,
         # introduced in numpy v1.22+, instead just including the unit info as
         # metadata using Annotated.
         # TODO: ensure we do interact with NDArray.__class_getitem__.
-        return Annotated.__class_getitem__((cls, unit))
+        return Annotated[cls, unit]
 
     def __new__(
         cls,
@@ -593,17 +582,20 @@ class Quantity(np.ndarray):
         if obj is None or obj.__class__ is np.ndarray:
             return
 
-        # If our unit is not set and obj has a valid one, use it.
+        # Copy over the unit and possibly info.  Note that the only way the
+        # unit can already be set is if one enters via _new_view(), where the
+        # unit is often different from that of self, and where propagation of
+        # info is not always desirable.
         if self._unit is None:
             unit = getattr(obj, "_unit", None)
             if unit is not None:
                 self._set_unit(unit)
 
-        # Copy info if the original had `info` defined.  Because of the way the
-        # DataInfo works, `'info' in obj.__dict__` is False until the
-        # `info` attribute is accessed or set.
-        if "info" in obj.__dict__:
-            self.info = obj.info
+            # Copy info if the original had `info` defined.  Because of the way the
+            # DataInfo works, `'info' in obj.__dict__` is False until the
+            # `info` attribute is accessed or set.
+            if "info" in obj.__dict__:
+                self.info = obj.info
 
     def __array_wrap__(self, obj, context=None):
         if context is None:
@@ -633,53 +625,70 @@ class Quantity(np.ndarray):
 
         Returns
         -------
-        result : `~astropy.units.Quantity`
+        result : `~astropy.units.Quantity` or `NotImplemented`
             Results of the ufunc, with the unit set properly.
         """
         # Determine required conversion functions -- to bring the unit of the
         # input to that expected (e.g., radian for np.sin), or to get
         # consistent units between two inputs (e.g., in np.add) --
         # and the unit of the result (or tuple of units for nout > 1).
-        converters, unit = converters_and_unit(function, method, *inputs)
+        try:
+            converters, unit = converters_and_unit(function, method, *inputs)
 
-        out = kwargs.get("out", None)
-        # Avoid loop back by turning any Quantity output into array views.
-        if out is not None:
-            # If pre-allocated output is used, check it is suitable.
-            # This also returns array view, to ensure we don't loop back.
-            if function.nout == 1:
-                out = out[0]
-            out_array = check_output(out, unit, inputs, function=function)
-            # Ensure output argument remains a tuple.
-            kwargs["out"] = (out_array,) if function.nout == 1 else out_array
+            out = kwargs.get("out", None)
+            # Avoid loop back by turning any Quantity output into array views.
+            if out is not None:
+                # If pre-allocated output is used, check it is suitable.
+                # This also returns array view, to ensure we don't loop back.
+                if function.nout == 1:
+                    out = out[0]
+                out_array = check_output(out, unit, inputs, function=function)
+                # Ensure output argument remains a tuple.
+                kwargs["out"] = (out_array,) if function.nout == 1 else out_array
 
-        if method == "reduce" and "initial" in kwargs and unit is not None:
-            # Special-case for initial argument for reductions like
-            # np.add.reduce.  This should be converted to the output unit as
-            # well, which is typically the same as the input unit (but can
-            # in principle be different: unitless for np.equal, radian
-            # for np.arctan2, though those are not necessarily useful!)
-            kwargs["initial"] = self._to_own_unit(
-                kwargs["initial"], check_precision=False, unit=unit
+            if method == "reduce" and "initial" in kwargs and unit is not None:
+                # Special-case for initial argument for reductions like
+                # np.add.reduce.  This should be converted to the output unit as
+                # well, which is typically the same as the input unit (but can
+                # in principle be different: unitless for np.equal, radian
+                # for np.arctan2, though those are not necessarily useful!)
+                kwargs["initial"] = self._to_own_unit(
+                    kwargs["initial"], check_precision=False, unit=unit
+                )
+
+            # Same for inputs, but here also convert if necessary.
+            arrays = []
+            for input_, converter in zip(inputs, converters):
+                input_ = getattr(input_, "value", input_)
+                arrays.append(converter(input_) if converter else input_)
+
+            # Call our superclass's __array_ufunc__
+            result = super().__array_ufunc__(function, method, *arrays, **kwargs)
+            # If unit is None, a plain array is expected (e.g., comparisons), which
+            # means we're done.
+            # We're also done if the result was None (for method 'at') or
+            # NotImplemented, which can happen if other inputs/outputs override
+            # __array_ufunc__; hopefully, they can then deal with us.
+            if unit is None or result is None or result is NotImplemented:
+                return result
+
+            return self._result_as_quantity(result, unit, out)
+
+        except (TypeError, ValueError, AttributeError) as e:
+            out_normalized = kwargs.get("out", tuple())
+            inputs_and_outputs = inputs + out_normalized
+            ignored_ufunc = (
+                None,
+                np.ndarray.__array_ufunc__,
+                type(self).__array_ufunc__,
             )
-
-        # Same for inputs, but here also convert if necessary.
-        arrays = []
-        for input_, converter in zip(inputs, converters):
-            input_ = getattr(input_, "value", input_)
-            arrays.append(converter(input_) if converter else input_)
-
-        # Call our superclass's __array_ufunc__
-        result = super().__array_ufunc__(function, method, *arrays, **kwargs)
-        # If unit is None, a plain array is expected (e.g., comparisons), which
-        # means we're done.
-        # We're also done if the result was None (for method 'at') or
-        # NotImplemented, which can happen if other inputs/outputs override
-        # __array_ufunc__; hopefully, they can then deal with us.
-        if unit is None or result is None or result is NotImplemented:
-            return result
-
-        return self._result_as_quantity(result, unit, out)
+            if not all(
+                getattr(type(io), "__array_ufunc__", None) in ignored_ufunc
+                for io in inputs_and_outputs
+            ):
+                return NotImplemented
+            else:
+                raise e
 
     def _result_as_quantity(self, result, unit, out):
         """Turn result into a quantity with the given unit.
@@ -707,14 +716,21 @@ class Quantity(np.ndarray):
         if isinstance(result, (tuple, list)):
             if out is None:
                 out = (None,) * len(result)
-            return result.__class__(
+            # Some np.linalg functions return namedtuple, which is handy to access
+            # elements by name, but cannot be directly initialized with an iterator.
+            result_cls = getattr(result, "_make", result.__class__)
+            return result_cls(
                 self._result_as_quantity(result_, unit_, out_)
                 for (result_, unit_, out_) in zip(result, unit, out)
             )
 
         if out is None:
             # View the result array as a Quantity with the proper unit.
-            return result if unit is None else self._new_view(result, unit)
+            return (
+                result
+                if unit is None
+                else self._new_view(result, unit, propagate_info=False)
+            )
 
         elif isinstance(out, Quantity):
             # For given Quantity output, just set the unit. We know the unit
@@ -744,9 +760,8 @@ class Quantity(np.ndarray):
         """
         return Quantity, True
 
-    def _new_view(self, obj=None, unit=None):
-        """
-        Create a Quantity view of some array-like input, and set the unit
+    def _new_view(self, obj=None, unit=None, propagate_info=True):
+        """Create a Quantity view of some array-like input, and set the unit.
 
         By default, return a view of ``obj`` of the same class as ``self`` and
         with the same unit.  Subclasses can override the type of class for a
@@ -768,9 +783,15 @@ class Quantity(np.ndarray):
             subclass, and explicitly assigned to the view if given.
             If not given, the subclass and unit will be that of ``self``.
 
+        propagate_info : bool, optional
+            Whether to transfer ``info`` if present.  Default: `True`, as
+            appropriate for, e.g., unit conversions or slicing, where the
+            nature of the object does not change.
+
         Returns
         -------
         view : `~astropy.units.Quantity` subclass
+
         """
         # Determine the unit and quantity subclass that we need for the view.
         if unit is None:
@@ -807,13 +828,15 @@ class Quantity(np.ndarray):
         view = obj.view(quantity_subclass)
         view._set_unit(unit)
         view.__array_finalize__(self)
+        if propagate_info and "info" in self.__dict__:
+            view.info = self.info
         return view
 
     def _set_unit(self, unit):
         """Set the unit.
 
         This is used anywhere the unit is set or modified, i.e., in the
-        initilizer, in ``__imul__`` and ``__itruediv__`` for in-place
+        initializer, in ``__imul__`` and ``__itruediv__`` for in-place
         multiplication and division by another unit, as well as in
         ``__array_finalize__`` for wrapping up views.  For Quantity, it just
         sets the unit, but subclasses can override it to check that, e.g.,
@@ -902,7 +925,7 @@ class Quantity(np.ndarray):
             If `True` (default), then the value is copied.  Otherwise, a copy
             will only be made if necessary.
 
-        See also
+        See Also
         --------
         to_value : get the numerical value in a given unit.
         """
@@ -942,7 +965,7 @@ class Quantity(np.ndarray):
             The value in the units specified. For arrays, this will be a view
             of the data if no unit conversion was necessary.
 
-        See also
+        See Also
         --------
         to : Get a new instance in a different unit.
         """
@@ -988,7 +1011,6 @@ class Quantity(np.ndarray):
         A `~astropy.units.UnitBase` object representing the unit of this
         quantity.
         """
-
         return self._unit
 
     @property
@@ -997,7 +1019,6 @@ class Quantity(np.ndarray):
         A list of equivalencies that will be applied by default during
         unit conversions.
         """
-
         return self._equivalencies
 
     def _recursively_apply(self, func):
@@ -1186,10 +1207,11 @@ class Quantity(np.ndarray):
     # Arithmetic operations
     def __mul__(self, other):
         """Multiplication between `Quantity` objects and other objects."""
-
         if isinstance(other, (UnitBase, str)):
             try:
-                return self._new_view(self.copy(), other * self.unit)
+                return self._new_view(
+                    self.value.copy(), other * self.unit, propagate_info=False
+                )
             except UnitsError:  # let other try to deal with it
                 return NotImplemented
 
@@ -1197,7 +1219,6 @@ class Quantity(np.ndarray):
 
     def __imul__(self, other):
         """In-place multiplication between `Quantity` objects and others."""
-
         if isinstance(other, (UnitBase, str)):
             self._set_unit(other * self.unit)
             return self
@@ -1208,15 +1229,15 @@ class Quantity(np.ndarray):
         """
         Right Multiplication between `Quantity` objects and other objects.
         """
-
         return self.__mul__(other)
 
     def __truediv__(self, other):
         """Division between `Quantity` objects and other objects."""
-
         if isinstance(other, (UnitBase, str)):
             try:
-                return self._new_view(self.copy(), self.unit / other)
+                return self._new_view(
+                    self.value.copy(), self.unit / other, propagate_info=False
+                )
             except UnitsError:  # let other try to deal with it
                 return NotImplemented
 
@@ -1224,7 +1245,6 @@ class Quantity(np.ndarray):
 
     def __itruediv__(self, other):
         """Inplace division between `Quantity` objects and other objects."""
-
         if isinstance(other, (UnitBase, str)):
             self._set_unit(self.unit / other)
             return self
@@ -1233,16 +1253,19 @@ class Quantity(np.ndarray):
 
     def __rtruediv__(self, other):
         """Right Division between `Quantity` objects and other objects."""
-
         if isinstance(other, (UnitBase, str)):
-            return self._new_view(1.0 / self.value, other / self.unit)
+            return self._new_view(
+                1.0 / self.value, other / self.unit, propagate_info=False
+            )
 
         return super().__rtruediv__(other)
 
     def __pow__(self, other):
         if isinstance(other, Fraction):
             # Avoid getting object arrays by raising the value to a Fraction.
-            return self._new_view(self.value ** float(other), self.unit**other)
+            return self._new_view(
+                self.value ** float(other), self.unit**other, propagate_info=False
+            )
 
         return super().__pow__(other)
 
@@ -1266,7 +1289,9 @@ class Quantity(np.ndarray):
 
     def __getitem__(self, key):
         if isinstance(key, str) and isinstance(self.unit, StructuredUnit):
-            return self._new_view(self.view(np.ndarray)[key], self.unit[key])
+            return self._new_view(
+                self.view(np.ndarray)[key], self.unit[key], propagate_info=False
+            )
 
         try:
             out = super().__getitem__(key)
@@ -1303,15 +1328,13 @@ class Quantity(np.ndarray):
     # __contains__ is OK
 
     def __bool__(self):
-        """Quantities should always be treated as non-False; there is too much
-        potential for ambiguity otherwise.
+        """This method raises ValueError, since truthiness of quantities is ambiguous,
+        especially for logarithmic units and temperatures. Use explicit comparisons.
         """
-        warnings.warn(
-            "The truth value of a Quantity is ambiguous. "
-            "In the future this will raise a ValueError.",
-            AstropyDeprecationWarning,
+        raise ValueError(
+            f"{type(self).__name__} truthiness is ambiguous, especially for logarithmic units"
+            " and temperatures. Use explicit comparisons."
         )
-        return True
 
     def __len__(self):
         if self.isscalar:
@@ -1572,7 +1595,6 @@ class Quantity(np.ndarray):
             A new object equal to this quantity with units decomposed.
 
         """
-
         new_unit = self.unit.decompose(bases=bases)
 
         # Be careful here because self.value usually is a view of self;
@@ -1614,7 +1636,7 @@ class Quantity(np.ndarray):
         value : anything convertible to `~astropy.units.Quantity`
             The value to be converted to the requested unit.
         check_precision : bool
-            Whether to forbit conversion of float to integer if that changes
+            Whether to forbid conversion of float to integer if that changes
             the input number.  Default: `True`.
         unit : `~astropy.units.Unit` or None
             The unit to convert to.  By default, the unit of ``self``.
@@ -1668,8 +1690,9 @@ class Quantity(np.ndarray):
 
         # Setting names to ensure things like equality work (note that
         # above will have failed already if units did not match).
-        if self.dtype.names:
-            _value.dtype.names = self.dtype.names
+        # TODO: is this the best place to do this?
+        if _value.dtype.names is not None:
+            _value = _value.astype(self.dtype, copy=False)
         return _value
 
     def itemset(self, *args):
@@ -1990,12 +2013,13 @@ class Quantity(np.ndarray):
 
     if NUMPY_LT_1_22:
 
+        @deprecated("5.3", alternative="np.nansum", obj_type="method")
         def nansum(self, axis=None, out=None, keepdims=False):
             return self._wrap_function(np.nansum, axis, out=out, keepdims=keepdims)
 
     else:
-        # TODO: deprecate this method? It is not on ndarray, and we do not
-        # support nanmean, etc., so why this one?
+
+        @deprecated("5.3", alternative="np.nansum", obj_type="method")
         def nansum(
             self, axis=None, out=None, keepdims=False, *, initial=None, where=True
         ):
@@ -2145,7 +2169,7 @@ def isclose(a, b, rtol=1.0e-5, atol=None, equal_nan=False, **kwargs):
         If the dimensions of ``a``, ``b``, or ``atol`` are incompatible,
         or if ``rtol`` is not dimensionless.
 
-    See also
+    See Also
     --------
     allclose
     """
@@ -2189,7 +2213,7 @@ def allclose(a, b, rtol=1.0e-5, atol=None, equal_nan=False, **kwargs) -> bool:
         If the dimensions of ``a``, ``b``, or ``atol`` are incompatible,
         or if ``rtol`` is not dimensionless.
 
-    See also
+    See Also
     --------
     isclose
     """
