@@ -11,6 +11,7 @@ from numpy.testing import assert_array_equal
 
 from astropy import units as u
 from astropy.units import Quantity
+from astropy.utils.compat.numpycompat import NUMPY_LT_1_25
 from astropy.utils.masked.core import Masked
 
 from .test_masked import (
@@ -25,24 +26,74 @@ class MaskedUfuncTests(MaskedArraySetup):
     @pytest.mark.parametrize(
         "ufunc", (np.add, np.subtract, np.divide, np.arctan2, np.minimum)
     )
-    def test_2op_ufunc(self, ufunc):
-        ma_mb = ufunc(self.ma, self.mb)
+    @pytest.mark.parametrize("a, b", [("ma", "mb"), ("ma", "b"), ("a", "mb")])
+    def test_2op_ufunc(self, ufunc, a, b):
+        a, b = getattr(self, a), getattr(self, b)
+        mask_a = getattr(a, "mask", np.zeros(a.shape, bool))
+        mask_b = getattr(b, "mask", np.zeros(b.shape, bool))
+        result = ufunc(a, b)
         expected_data = ufunc(self.a, self.b)
-        expected_mask = self.ma.mask | self.mb.mask
+        expected_mask = mask_a | mask_b
         # Note: assert_array_equal also checks type, i.e., that, e.g.,
         # Longitude decays into an Angle.
-        assert_array_equal(ma_mb.unmasked, expected_data)
-        assert_array_equal(ma_mb.mask, expected_mask)
+        assert_array_equal(result.unmasked, expected_data)
+        assert_array_equal(result.mask, expected_mask)
 
-    @pytest.mark.parametrize(
-        "ufunc", (np.add, np.subtract, np.divide, np.arctan2, np.minimum)
-    )
-    def test_ufunc_inplace(self, ufunc):
-        ma_mb = ufunc(self.ma, self.mb)
-        out = Masked(np.zeros_like(ma_mb.unmasked))
-        result = ufunc(self.ma, self.mb, out=out)
-        assert result is out
-        assert_masked_equal(result, ma_mb)
+        out = Masked(np.zeros_like(result.unmasked))
+        result2 = ufunc(a, b, out=out)
+        assert result2 is out
+        assert_masked_equal(result2, result)
+
+    @pytest.mark.parametrize("base_mask", [True, False])
+    def test_ufunc_inplace_where(self, base_mask):
+        # Construct base filled with -9 and base_mask (copying to get unit/class).
+        base = self.ma.copy()
+        base.unmasked.view(np.ndarray)[...] = -9.0
+        base._mask[...] = base_mask
+        out = base.copy()
+        where = np.array([[True, False, False], [False, True, False]])
+        result = np.add(self.ma, self.mb, out=out, where=where)
+        # Direct checks.
+        assert np.all(result.unmasked[~where] == base.unmasked[0, 0])
+        assert np.all(result.unmasked[where] == (self.a + self.b)[where])
+        # Full comparison.
+        expected = base.unmasked.copy()
+        np.add(self.a, self.b, out=expected, where=where)
+        expected_mask = base.mask.copy()
+        np.logical_or(self.mask_a, self.mask_b, out=expected_mask, where=where)
+        assert_array_equal(result.unmasked, expected)
+        assert_array_equal(result.mask, expected_mask)
+
+    @pytest.mark.parametrize("base_mask", [True, False])
+    def test_ufunc_inplace_masked_where(self, base_mask):
+        base = self.ma.copy()
+        base.unmasked.view(np.ndarray)[...] = -9.0
+        base._mask[...] = base_mask
+        out = base.copy()
+        where = Masked(
+            [[True, False, True], [False, False, True]],
+            mask=[[True, False, False], [True, False, True]],
+        )
+        result = np.add(self.ma, self.mb, out=out, where=where)
+        # Direct checks.
+        assert np.all(result.unmasked[~where.unmasked] == base.unmasked[0, 0])
+        assert np.all(
+            result.unmasked[where.unmasked] == (self.a + self.b)[where.unmasked]
+        )
+        assert np.all(result.mask[where.mask])
+        assert np.all(result.mask[~where.mask & ~where.unmasked] == base.mask[0, 0])
+        assert np.all(
+            result.mask[~where.mask & where.unmasked]
+            == (self.mask_a | self.mask_b)[~where.mask & where.unmasked]
+        )
+        # Full comparison.
+        expected = base.unmasked.copy()
+        np.add(self.a, self.b, out=expected, where=where.unmasked)
+        expected_mask = base.mask.copy()
+        np.logical_or(self.mask_a, self.mask_b, out=expected_mask, where=where.unmasked)
+        expected_mask |= where.mask
+        assert_array_equal(result.unmasked, expected)
+        assert_array_equal(result.mask, expected_mask)
 
     def test_ufunc_inplace_no_masked_input(self):
         a_b = np.add(self.a, self.b)
@@ -53,19 +104,45 @@ class MaskedUfuncTests(MaskedArraySetup):
         assert_array_equal(result.mask, np.zeros(a_b.shape, bool))
 
     def test_ufunc_inplace_error(self):
+        # Output is not masked.
         out = np.zeros(self.ma.shape)
         with pytest.raises(TypeError):
             np.add(self.ma, self.mb, out=out)
 
+    @pytest.mark.xfail(NUMPY_LT_1_25, reason="masked where not supported in numpy<1.25")
+    def test_ufunc_inplace_error_masked_where(self):
+        # Input and output are not masked, but where is.
+        # Note: prior to numpy 1.25, we cannot control this.
+        out = self.a.copy()
+        with pytest.raises(TypeError):
+            np.add(self.a, self.b, out=out, where=Masked(True, mask=True))
+
     @pytest.mark.parametrize("ufunc", (np.add.outer, np.minimum.outer))
-    def test_2op_ufunc_outer(self, ufunc):
-        ma_mb = ufunc(self.ma, self.mb)
+    @pytest.mark.parametrize("a, b", [("ma", "mb"), ("ma", "b"), ("a", "mb")])
+    def test_2op_ufunc_outer(self, ufunc, a, b):
+        a, b = getattr(self, a), getattr(self, b)
+        mask_a = getattr(a, "mask", np.zeros(a.shape, bool))
+        mask_b = getattr(b, "mask", np.zeros(b.shape, bool))
+        result = ufunc(a, b)
         expected_data = ufunc(self.a, self.b)
-        expected_mask = np.logical_or.outer(self.mask_a, self.mask_b)
+        expected_mask = np.logical_or.outer(mask_a, mask_b)
         # Note: assert_array_equal also checks type, i.e., that, e.g.,
         # Longitude decays into an Angle.
-        assert_array_equal(ma_mb.unmasked, expected_data)
-        assert_array_equal(ma_mb.mask, expected_mask)
+        assert_array_equal(result.unmasked, expected_data)
+        assert_array_equal(result.mask, expected_mask)
+
+        out = Masked(np.zeros_like(result.unmasked))
+        result2 = ufunc(a, b, out=out)
+        assert result2 is out
+        assert_masked_equal(result2, result)
+
+    @pytest.mark.parametrize("ufunc", (np.add.outer, np.minimum.outer))
+    def test_2op_ufunc_outer_no_masked_input(self, ufunc):
+        expected_data = ufunc(self.a, self.b)
+        out = Masked(np.zeros_like(expected_data), True)
+        result = ufunc(self.a, self.b, out=out)
+        assert_array_equal(out.unmasked, expected_data)
+        assert_array_equal(out.mask, np.zeros(out.shape, dtype=bool))
 
     def test_3op_ufunc(self):
         ma_mb = np.clip(self.ma, self.b, self.c)
